
;**************************************************************************
;*                    Operation-Hour Counter                              *
;**************************************************************************
;**************************************************************************
;*                      v 1.13 - 13.09.2015                               *
;*                                                                        *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F1828 used                                       *
;*          OSC.......: Ext. 4MHz Quarz        POWER.....:   5V extern.   *
;**************************************************************************


 INCLUDE "P16F1828.INC"


_U001			     EQU     B'00100110001010'		;see Page 48 of Datasheet

	;HS oscillator (bit2-0 = 010)
	;MCLR disabled
	;WDT enabled and controlled by SWDTEN bit of register WDTCON (bit4-3 = 01)
	;no protection
	;BOR disabled


_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	;write protection off
	;PLL enabled



	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002



;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_ttt		EQU	H'22'
data_tmp	EQU	H'23'
c_tmp		EQU	H'24'
kbd_buffer	EQU	H'25'
v_tst1		EQU	H'26'
v_tst2		EQU	H'27'
v_tst3		EQU	H'28'
disp_pos	EQU	H'29'

v_irtc		EQU	H'2A'
v_isf		EQU	H'2B'
v_t_corr	EQU	H'2C'

;----------------------------
v_cnt1_0	EQU	H'30'
v_cnt1_1	EQU	H'31'
v_cnt1_2	EQU	H'32'
v_cnt1_3	EQU	H'33'
v_cnt1_4	EQU	H'34'
v_cnt1_5	EQU	H'35'

v_cnt1_m	EQU	H'36'
v_cnt1_s1	EQU	H'37'
v_cnt1_s2	EQU	H'38'

v_cnt1_on	EQU	H'39'
;----------------------------

;----------------------------
v_cnt2_0	EQU	H'40'
v_cnt2_1	EQU	H'41'
v_cnt2_2	EQU	H'42'
v_cnt2_3	EQU	H'43'
v_cnt2_4	EQU	H'44'
v_cnt2_5	EQU	H'45'

v_cnt2_m	EQU	H'46'
v_cnt2_s1	EQU	H'47'
v_cnt2_s2	EQU	H'48'

v_cnt2_on	EQU	H'49'
;----------------------------



v_h1		EQU	H'50'
v_h2		EQU	H'51'
v_m1		EQU	H'52'
v_m2		EQU	H'53'
v_s1		EQU	H'54'
v_s2		EQU	H'55'

v_power_loss	EQU	H'56'
;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

		ORG	4
		goto	ir_service	;interrupt service routine

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	nop
		clrf	BSR

		clrf	INTCON			;disable interrupts

		movlw	H'FF'
		movwf	PORTB

		banksel	OPTION_REG	;BANK 1
		movlw	B'01111111'
		movwf	OPTION_REG	;see datasheet Page 176

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA
		clrf	ANSELB
		clrf	ANSELC

		movlw	H'01'
		movwf	BSR		;select BANK 1

		movlw	B'11111111'
		movwf	TRISA

		movlw	B'00111111'
		movwf	TRISB

		movlw	B'00000000'
		movwf	TRISC

;no changes required to OSCCON as after RESET the value from Config 1 word will be used (OSCCON - bit 1:bit 0 = 00)

		movlw	H'04'
		movwf	BSR		;select BANK 4
		clrf	WPUA		;disable weak pull-ups for PORTA
		clrf	WPUB		;disable weak pull-ups for PORTB
		clrf	WPUC		;disable weak pull-ups for PORTC
		bsf	WPUA,D'000'	;enable week pull-up for RA0
		bsf	WPUA,D'001'	;enable week pull-up for RA1
		bsf	WPUA,D'002'	;enable week pull-up for RA2

		movlw	H'07'
		movwf	BSR		;select BANK 7
		movlw	H'FF'
		movwf	INLVLA		;ST (1) level or TTL (0) level (?)
		movwf	INLVLB		;ST (1) level or TTL (0) level (?)
		movwf	INLVLC		;ST (1) level or TTL (0) level (?)

		banksel	LATC
		clrf	LATA
		clrf	LATC

		clrf	BSR		;select BANK 0

;comment: no need to change the WDTCON register
;- as reset-value set to WDT to 2 seconds - what is ok for the application
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		movlw	H'FF'
		movwf	PORTB

		call	disp_init
		call	w_release

		call	dr4

		call	disp_clr1
		call	disp_clr2

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		banksel	WDTCON
		bsf	WDTCON,D'000'		;enable Watch-dog
		clrf	BSR			;select BANK 0

		call	timer2_init

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		goto	main_prg


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Interrupt service routine
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

ir_service	clrf	BSR

		btfsc	v_power_loss,D'000'
		goto	irs_fin2

		bcf	INTCON,GIE

		clrwdt				;clear Watchdog timer

		incf	v_irtc,1
		movf	v_irtc,0
		xorlw	D'125'
		btfss	STATUS,Z
		goto	irs_fin

;1 sec is over
		clrf	v_irtc
		bsf	v_isf,D'000'

		incf	v_t_corr,1
		movf	v_t_corr,0
		xorlw	D'060'
		btfss	STATUS,Z
		goto	irs_fin
;---------------------------------------------------------------------------
;clock is a little bit too slow; therefore every 60 seconds we do a small correction
		movlw	D'018'
		movwf	v_irtc
		clrf	v_t_corr
;---------------------------------------------------------------------------
irs_fin		btfss	PORTA,D'002'
		call	power_loss

irs_fin2	clrf	TMR2
		bcf	PIR1,TMR2IF

		bsf	INTCON,GIE

		retfie
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
power_loss	bsf	v_power_loss,D'000'
		call	ee_write_cnt1
		call	ee_write_cnt2

		call	w_pl

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main entry screen
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

main_prg	bcf	INTCON,GIE
		clrf	v_power_loss

		btfss	PORTA,D'002'
		call	w_pu
mp_wp		btfss	PORTA,D'002'
		goto	mp_wp

		call	disp_clr1
		call	disp_clr2

		call	init_int_clk

;start the time counter ----------------
		bsf	INTCON,GIE
		clrf	v_irtc
		clrf	v_isf
		bsf	T2CON,D'002'
;---------------------------------------

		clrf	v_cnt1_on
		clrf	v_cnt2_on

		call	ee_read_cnt1		;read counter 1 from EEPROM
		call	ee_read_cnt2		;read counter 2 from EEPROM

main_loop	nop

		btfss	PORTA,D'000'
		call	cnt1_off
		btfsc	PORTA,D'000'
		call	cnt1_on

		btfss	PORTA,D'001'
		call	cnt2_off
		btfsc	PORTA,D'001'
		call	cnt2_on

		call	inc_time_2
		call	w_timers

;autosave every 10 hours -------------------------
		movf	v_h1,0
		xorlw	'1'
		btfsc	STATUS,Z
		call	ee_auto_save
;-------------------------------------------------

		btfss	PORTA,D'002'
		goto	main_prg

		goto	main_loop
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;save the counter to EEPROM - just for sure... - every 10 hours...
;
ee_auto_save	call	w_cnt1_wi
		call	ee_write_cnt1
		call	w_cnt2_wi
		call	ee_write_cnt2
		call	init_int_clk
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
cnt1_on		btfsc	v_cnt1_on,D'000'
		return				;counter 1 is running already
		bsf	v_cnt1_on,D'000'
		bsf	v_cnt1_on,D'001'
		bcf	PORTB,D'006'		;switch on the device connected to RB6 (active level = 0)
		return
;------------------------------------------------------------------------------
cnt2_on		btfsc	v_cnt2_on,D'000'
		return				;counter 2 is running already
		bsf	v_cnt2_on,D'000'
		bsf	v_cnt2_on,D'001'
		bcf	PORTB,D'007'		;switch on the device connected to RB7 (active level = 0)
		return
;------------------------------------------------------------------------------
cnt1_off	bcf	v_cnt1_on,D'000'
		btfss	v_cnt1_on,D'001'
		return
		call	w_cnt1_wi
		call	ee_write_cnt1		;write counter 1 to EEPROM
		bcf	v_cnt1_on,D'001'
		bsf	PORTB,D'006'		;switch off the device connected to RB6 (inactive level = 1)
		return
;------------------------------------------------------------------------------
cnt2_off	bcf	v_cnt2_on,D'000'
		btfss	v_cnt2_on,D'001'
		return
		call	w_cnt2_wi
		call	ee_write_cnt2		;write counter 2 to EEPROM
		bcf	v_cnt2_on,D'001'
		bsf	PORTB,D'007'		;switch off the device connected to RB7 (inactive level = 1)
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; E E P R O M
;
; Data Organization:
;   00h - 08h - Counter 1 Data
;   09h - 0Fh - not used / reserved for Counter 1
;   10h - 18h - Counter 2 Data
;   19h - 1Fh - not used / reserved for Counter 2
;
;
;   Counter Data Format: 00.000,0   0:00
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ee_w_byte	bcf	EECON1,CFGS
		bcf	EECON1,EEPGD
		bsf	EECON1,WREN

		bcf	INTCON,GIE
		movlw	H'55'
		movwf	EECON2
		movlw	H'AA'
		movwf	EECON2
		bsf	EECON1,WR
		btfss	v_power_loss,D'000'		;not interrupt enable in power loss mode
		bsf	INTCON,GIE
		bsf	EECON1,WREN
eewb_l1		btfsc	EECON1,WR
		goto	eewb_l1

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ee_write_cnt1	movf	v_cnt1_0,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'00'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt1_0
		movf	v_cnt1_1,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'01'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt1_0
		movf	v_cnt1_2,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'02'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt1_0
		movf	v_cnt1_3,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'03'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt1_0
		movf	v_cnt1_4,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'04'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt1_0
		movf	v_cnt1_5,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'05'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt1_0
		movf	v_cnt1_m,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'06'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt1_0
		movf	v_cnt1_s1,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'07'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt1_0
		movf	v_cnt1_s2,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'08'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt1_0
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ee_write_cnt2	movf	v_cnt2_0,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'10'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt2_0
		movf	v_cnt2_1,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'11'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt2_0
		movf	v_cnt2_2,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'12'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt2_0
		movf	v_cnt2_3,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'13'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt2_0
		movf	v_cnt2_4,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'14'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt2_0
		movf	v_cnt2_5,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'15'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt2_0
		movf	v_cnt2_m,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'16'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt2_0
		movf	v_cnt2_s1,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'17'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt2_0
		movf	v_cnt2_s2,0
		banksel	EEADRL
		movwf	EEDATL
		movlw	H'18'
		movwf	EEADRL
		call	ee_w_byte

		banksel	v_cnt2_0
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ee_read_cnt1	nop
		banksel	EEADRL
		movlw	H'00'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt1_0
		movwf	v_cnt1_0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	init_cnt1		;nothing found in EEPROM

		banksel	EEADRL
		movlw	H'01'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt1_0
		movwf	v_cnt1_1

		banksel	EEADRL
		movlw	H'02'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt1_0
		movwf	v_cnt1_2

		banksel	EEADRL
		movlw	H'03'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt1_0
		movwf	v_cnt1_3

		banksel	EEADRL
		movlw	H'04'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt1_0
		movwf	v_cnt1_4

		banksel	EEADRL
		movlw	H'05'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt1_0
		movwf	v_cnt1_5

		banksel	EEADRL
		movlw	H'06'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt1_0
		movwf	v_cnt1_m

		banksel	EEADRL
		movlw	H'07'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt1_0
		movwf	v_cnt1_s1

		banksel	EEADRL
		movlw	H'08'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt1_0
		movwf	v_cnt1_s2

		return
;------------------------------------------------------------------------------
ee_read_cnt2	nop
		banksel	EEADRL
		movlw	H'10'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt2_0
		movwf	v_cnt2_0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	init_cnt2		;nothing found in EEPROM

		banksel	EEADRL
		movlw	H'11'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt2_0
		movwf	v_cnt2_1

		banksel	EEADRL
		movlw	H'12'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt2_0
		movwf	v_cnt2_2

		banksel	EEADRL
		movlw	H'13'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt2_0
		movwf	v_cnt2_3

		banksel	EEADRL
		movlw	H'14'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt2_0
		movwf	v_cnt2_4

		banksel	EEADRL
		movlw	H'15'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt2_0
		movwf	v_cnt2_5

		banksel	EEADRL
		movlw	H'16'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt2_0
		movwf	v_cnt2_m

		banksel	EEADRL
		movlw	H'17'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt2_0
		movwf	v_cnt2_s1

		banksel	EEADRL
		movlw	H'18'
		movwf	EEADRL
		call	ee_read_byte
		banksel	v_cnt2_0
		movwf	v_cnt2_s2

		return
;------------------------------------------------------------------------------
ee_read_byte	bcf	EECON1,CFGS
		bcf	EECON1,EEPGD
		bsf	EECON1,RD
		movf	EEDATL,0
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
init_cnt1	movlw	'0'
		movwf	v_cnt1_0
		movwf	v_cnt1_1
		movwf	v_cnt1_2
		movwf	v_cnt1_3
		movwf	v_cnt1_4
		movwf	v_cnt1_5

		movwf	v_cnt1_m
		movwf	v_cnt1_s1
		movwf	v_cnt1_s2

		return
;------------------------------------------------
init_cnt2	movlw	'0'
		movwf	v_cnt2_0
		movwf	v_cnt2_1
		movwf	v_cnt2_2
		movwf	v_cnt2_3
		movwf	v_cnt2_4
		movwf	v_cnt2_5

		movwf	v_cnt2_m
		movwf	v_cnt2_s1
		movwf	v_cnt2_s2
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
init_int_clk	movlw	'0'
		movwf	v_h1
		movwf	v_h2
		movwf	v_m1
		movwf	v_m2
		movwf	v_s1
		movwf	v_s2
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
inc_time_2	btfss	v_isf,D'000'
		return

		clrf	v_isf

		call	cnt1_inc
		call	cnt2_inc
		call	inc_time_int
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
cnt1_inc	btfss	v_cnt1_on,D'000'
		return

		incf	v_cnt1_s2,1
		movf	v_cnt1_s2,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt1_s2

		incf	v_cnt1_s1,1
		movf	v_cnt1_s1,0
		xorlw	'6'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt1_s1

		incf	v_cnt1_m,1
		movf	v_cnt1_m,0
		xorlw	'6'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt1_m
;-----------------------------------
		incf	v_cnt1_5,1
		movf	v_cnt1_5,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt1_5

		incf	v_cnt1_4,1
		movf	v_cnt1_4,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt1_4

		incf	v_cnt1_3,1
		movf	v_cnt1_3,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt1_3

		incf	v_cnt1_2,1
		movf	v_cnt1_2,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt1_2

		incf	v_cnt1_1,1
		movf	v_cnt1_1,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt1_1

		incf	v_cnt1_0,1
		movf	v_cnt1_0,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt1_0

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
cnt2_inc	btfss	v_cnt2_on,D'000'
		return

		incf	v_cnt2_s2,1
		movf	v_cnt2_s2,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt2_s2

		incf	v_cnt2_s1,1
		movf	v_cnt2_s1,0
		xorlw	'6'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt2_s1

		incf	v_cnt2_m,1
		movf	v_cnt2_m,0
		xorlw	'6'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt2_m
;-----------------------------------

		incf	v_cnt2_5,1
		movf	v_cnt2_5,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt2_5

		incf	v_cnt2_4,1
		movf	v_cnt2_4,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt2_4

		incf	v_cnt2_3,1
		movf	v_cnt2_3,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt2_3

		incf	v_cnt2_2,1
		movf	v_cnt2_2,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt2_2

		incf	v_cnt2_1,1
		movf	v_cnt2_1,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt2_1

		incf	v_cnt2_0,1
		movf	v_cnt2_0,0
		xorlw	':'
		btfss	STATUS,Z
		return
		movlw	'0'
		movwf	v_cnt2_0

		return

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

inc_time	btfss	v_isf,D'000'
		return

		clrf	v_isf

inc_time_int	incf	v_s2,1
		movf	v_s2,0
		xorlw	':'
		btfss	STATUS,Z
		return

		movlw	'0'
		movwf	v_s2
		incf	v_s1,1
		movf	v_s1,0
		xorlw	'6'
		btfss	STATUS,Z
		return

		movlw	'0'
		movwf	v_s1
		incf	v_m2,1
		movf	v_m2,0
		xorlw	':'
		btfss	STATUS,Z
		return

		movlw	'0'
		movwf	v_m2
		incf	v_m1,1
		movf	v_m1,0
		xorlw	'6'
		btfss	STATUS,Z
		return

		movlw	'0'
		movwf	v_m1
		incf	v_h2,1
		movf	v_h2,0
		xorlw	':'
		btfss	STATUS,Z
		return

		movlw	'0'
		movwf	v_h2
		incf	v_h1,1
		movf	v_h1,0
		xorlw	':'
		btfss	STATUS,Z
		return

		movlw	'0'
		movwf	v_h1
		return
;------------------------------------------------------------
;------------------------------------------------------------
w_pu		movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_o
		call	wm_w
		call	wm_e
		call	wm_r
		call	wm_space
		call	wm_i
		call	wm_n
		call	wm_s
		call	wm_t
		call	wm_a
		call	wm_b
		call	wm_l
		call	wm_e
		return

;------------------------------------------------------------
;------------------------------------------------------------
w_pl		call	disp_clr1
		call	disp_clr2

		movlw	B'10000000'
		call	send_comm

		call	wm_P
		call	wm_o
		call	wm_w
		call	wm_e
		call	wm_r
		call	wm_space
		call	wm_l
		call	wm_o
		call	wm_s
		call	wm_s
		call	wm_space
		call	wm_d
		call	wm_e
		call	wm_t
		call	wm_e
		call	wm_c
		call	wm_t

		movlw	B'11000000'
		call	send_comm

		call	wm_D
		call	wm_a
		call	wm_t
		call	wm_a
		call	wm_space
		call	wm_s
		call	wm_a
		call	wm_v
		call	wm_e
		call	wm_d
		return
;------------------------------------------------------------
;------------------------------------------------------------
w_cnt1_wi	movlw	B'10000100'
		call	send_comm

		call	wm_S
		call	wm_a
		call	wm_v
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		return
;------------------------------------------------------------
w_cnt2_wi	movlw	B'11000100'
		call	send_comm

		call	wm_S
		call	wm_a
		call	wm_v
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		return
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
w_timers	movlw	B'10000000'
		call	send_comm

		call	wm_T
		call	wm_1
		call	wm_dvojbodka
		call	wm_space

		movf	v_cnt1_0,0
		call	send_data
		movf	v_cnt1_1,0
		call	send_data
		call	wm_bodka
		movf	v_cnt1_2,0
		call	send_data
		movf	v_cnt1_3,0
		call	send_data
		movf	v_cnt1_4,0
		call	send_data
		movlw	','
		call	send_data
		movf	v_cnt1_5,0
		call	send_data

		call	wm_space
		call	wm_space
		call	wm_space
		movf	v_cnt1_m,0
		call	send_data
		call	wm_dvojbodka
		movf	v_cnt1_s1,0
		call	send_data
		movf	v_cnt1_s2,0
		call	send_data

		movlw	B'11000000'
		call	send_comm

		call	wm_T
		call	wm_2
		call	wm_dvojbodka
		call	wm_space

		movf	v_cnt2_0,0
		call	send_data
		movf	v_cnt2_1,0
		call	send_data
		call	wm_bodka
		movf	v_cnt2_2,0
		call	send_data
		movf	v_cnt2_3,0
		call	send_data
		movf	v_cnt2_4,0
		call	send_data
		movlw	','
		call	send_data
		movf	v_cnt2_5,0
		call	send_data

		call	wm_space
		call	wm_space
		call	wm_space
		movf	v_cnt2_m,0
		call	send_data
		call	wm_dvojbodka
		movf	v_cnt2_s1,0
		call	send_data
		movf	v_cnt2_s2,0
		call	send_data

		call	cnt1_info
		call	cnt2_info

		return
;-------------------------------------------------------------------------------
cnt1_info	movlw	B'10010101'
		call	send_comm

		btfss	v_cnt1_on,D'000'
		call	cnt_off
		btfsc	v_cnt1_on,D'000'
		call	cnt_on
		return
;-------------------------------------------------------------------------------
cnt2_info	movlw	B'11010101'
		call	send_comm

		btfss	v_cnt2_on,D'000'
		call	cnt_off
		btfsc	v_cnt2_on,D'000'
		call	cnt_on
		return
;-------------------------------------------------------------------------------
cnt_on		call	wm_space
		call	wm_o
		call	wm_n
		return
;-------------------------------------------------------------------------------
cnt_off		call	wm_o
		call	wm_f
		call	wm_f
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_time		movlw	B'11000000'
		call	send_comm

		movf	v_h1,0
		call	send_data
		movf	v_h2,0
		call	send_data
		call	wm_dvojbodka

		movf	v_m1,0
		call	send_data
		movf	v_m2,0
		call	send_data
		call	wm_dvojbodka

		movf	v_s1,0
		call	send_data
		movf	v_s2,0
		call	send_data

		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x           LCD Display Communication Routines                               x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RE2 -- RA5 -- RC4
;R/W = RE1 -- N/C -- N/C
;E   = RE0 -- RA4 -- RC5

;D0 - D3 -- not connected

;D4  = RD0 -- RA0 -- RC0
;D5  = RD1 -- RA1 -- RC1
;D6  = RD2 -- RA2 -- RC2
;D7  = RD3 -- RA3 -- RC3


disp_init	nop

;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTC
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTC,D'004'
		bcf	PORTC,D'005'		;EN = 0 => disable


		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000010'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		movlw	B'00001100'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------

set_LCD_D	nop
		movwf	v_ttt

		btfss	v_ttt,D'000'
		bcf	PORTC,D'000'
		call	set_0

		btfss	v_ttt,D'001'
		bcf	PORTC,D'001'
		call	set_1

		btfss	v_ttt,D'002'
		bcf	PORTC,D'002'
		call	set_2

		btfss	v_ttt,D'003'
		bcf	PORTC,D'003'
		call	set_3
		return
;-------------------------------------------
set_0		btfsc	v_ttt,D'000'
		bsf	PORTC,D'000'
		return

set_1		btfsc	v_ttt,D'001'
		bsf	PORTC,D'001'
		return

set_2		btfsc	v_ttt,D'002'
		bsf	PORTC,D'002'
		return

set_3		btfsc	v_ttt,D'003'
		bsf	PORTC,D'003'
		return
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTC,D'005'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbd			;send

		return

;set control bits
scbd		bsf	PORTC,D'004'		;RS -> 1 (data)
		bsf	PORTC,D'005'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTC,D'005'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTC,D'004'		;RS -> 0 (command)
		bsf	PORTC,D'005'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		return

dy_comm2	MOVLW   D'010'			;delay value changed: original value was: 010
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		call	d1b		;02
		call	d1b		;02
		return

d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_hviezdicka	movlw	'*'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return

wm_and		movlw	'&'
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

disp_clrx	call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		goto	disp_clrx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
convert2ascii	movwf	c_tmp
;
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka

		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
;!!!!!!		goto	main_s


;not known character - either transmission error or not decoded... - do nothing...
		decf	disp_pos,1
		
		return


		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; Firmware Release
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-------------------------------------------------------------------------------
w_release	call	init2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_O
		call	wm_p
		call	wm_e
		call	wm_r
		call	wm_a
		call	wm_t
		call	wm_i
		call	wm_o
		call	wm_n
		call	wm_minus
		call	wm_H
		call	wm_o
		call	wm_u
		call	wm_r
		call	wm_space
		call	wm_C
		call	wm_o
		call	wm_u
		call	wm_n
		call	wm_t
		call	wm_e
		call	wm_r

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_F
		call	wm_i
		call	wm_r
		call	wm_m
		call	wm_w
		call	wm_a
		call	wm_r
		call	wm_e
		call	wm_space
		call	wm_1
		call	wm_bodka
		call	wm_1
		call	wm_3
		call	wm_minus
		call	wm_1
		call	wm_3
		call	wm_bodka
		call	wm_0
		call	wm_9
		call	wm_bodka
		call	wm_2
		call	wm_0
		call	wm_1
		call	wm_5

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_bin7	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		call	wm_x

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
write_bin	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'000'
		movlw	'1'
		call	send_data

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

conv_next	movf	kbd_buffer,0
		btfsc	STATUS,Z
		goto	tst_conv_f

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

wdcn		decf	kbd_buffer,1
		goto	conv_next
;-------------------------------------------------
tst_conv_f	movf	v_tst1,0
;;;		call	send_data		;do not write first digit
		movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return

;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
timer2_init	nop

;TIMER2 setup
		movlw	B'00100010'
		movwf	T2CON

			;prescaler  = 16 / <1:0> = 10		;TMR is inceremeted 62.500 x pro sec. (1MHz:16 = 62.500)
			;postscaler =  5 / <6:3> = 0100		;after "overflow" every 5 times is interrupt raised
			;timer if off    /  <2> = 0

		clrf	TMR2

		movlw	D'099'					;overflow after 100 pulses
		movwf	PR2

;with this settings interrupt should happen 125x per second


;Interrupt setup
		banksel	PIE1		

		clrf	PIE1
		clrf	PIE2
		clrf	PIE3

		bsf	PIE1,TMR2IE		;Enables the TMR2 to PR2 match interrupt

		clrf	BSR			;select BANK 0

		movlw	B'11000000'
		movwf	INTCON

		clrf	v_t_corr

;at this moment is everything prepared - but TIMER2 is not running

		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		END